#!/usr/bin/env python

# base imports
import sys
import tensorflow as tf

# user imports
import LoadCsvData
import LoadApacheLogData
import Data2DataRow
import LoadFeatureData
import LoadTensorData
import DataRow2Tensor
import RegModelLinear
import RegModelLogistic


dataDir = "./data"
appVersion = "0.4.0.7_python2.7"

def run(exeCfg):
    type = exeCfg['type']
    data_2_datarow_type = exeCfg['data_2_datarow_type']
    datarow_2_tensor_type = exeCfg['datarow_2_tensor_type']
    version = exeCfg['version']
    reset = exeCfg['reset']
    checkpoint = exeCfg['checkpoint']
    verbose = exeCfg['verbose']
    limitLoad = exeCfg['limitLoad']
    rowLimit = exeCfg['rowLimit']
    validatePrct = exeCfg['validatePrct']
    trainPrct = exeCfg['trainPrct']
    randomSeed = exeCfg['randomSeed']
    learning_rate = exeCfg['learning_rate']
    log_reg_positive_result = exeCfg['log_reg_positive_result']
    lin_reg_positive_result = exeCfg['lin_reg_positive_result']
    model_type = exeCfg['model_type']
    loader = exeCfg['loader']
    cleanData = exeCfg['cleanData']
    trainStepsMultiplier = exeCfg['trainStepsMultiplier']
    dataMap = Data2DataRow.mapping[data_2_datarow_type]
    files = exeCfg['files']
    featureType = exeCfg['feature_type']
    logPrint = exeCfg['logPrint']

    checkpointSave = 1000
    if 'checkpointSave' in exeCfg:
        checkpointSave = exeCfg['checkpointSave']
    # eif

    evalType = exeCfg['eval_type']
    data = None
    fData = None
    tData = None
    tfModel = None

    print("Found loader: " + loader)
    if loader == 'load_csv_data':
        data = LoadCsvData.LoadCsvData()
        data.limitLoad = limitLoad
        data.rowLimit = rowLimit
        data.verbose = verbose

        for file in files:
            csvFileName = files[file]['name']
            appendCols = files[file]['appendCols']
            data.loadData(csvFileName, type, version, reset, dataMap, appendCols)
        # efl

    elif loader == 'load_apache_log_data':
        data = LoadApacheLogData.LoadApacheLogData()
        data.limitLoad = limitLoad
        data.rowLimit = rowLimit
        data.verbose = verbose

        for file in files:
            logFileName = files[file]['name']
            appendCols = files[file]['appendCols']
            data.loadData(logFileName, type, version, reset, dataMap, appendCols)
        # efl

    # eif

    print("Found feature type: " + featureType)
    fData = LoadFeatureData.LoadFeatureData(limitLoad, rowLimit, cleanData, verbose, data)
    if 'extras' in exeCfg:
        fData.setExtras(exeCfg['extras'])
    # eif

    if featureType != '':
        fData.generateData(featureType)
    else:
        fData.generateData('')
    # eif

    tData = LoadTensorData.LoadTensorData(verbose, trainPrct, validatePrct, fData)
    tData.generateData(DataRow2Tensor.columns[datarow_2_tensor_type])

    if model_type == 'linear_regression':
        tfModel = RegModelLinear.RegModelLinear(verbose, tData, lin_reg_positive_result, randomSeed, trainStepsMultiplier, logPrint, learning_rate, evalType)
        tfModel.setCheckPointSave(checkpointSave)
        tfModel.setCheckPointOn(checkpoint)
        tfModel.startTraining()

    elif model_type == 'logistic_regression':
        tfModel = RegModelLogistic.RegModelLogistic(verbose, tData, log_reg_positive_result, randomSeed, trainStepsMultiplier, logPrint, learning_rate, evalType)
        tfModel.setCheckPointSave(checkpointSave)
        tfModel.setCheckPointOn(checkpoint)
        tfModel.startTraining()

    # eif

# edef

def main(args):

    print("Application Version: " + appVersion)

    exes = {
        "goog_lin_reg_avg100day":
        {
            'type': 'csv',
            'data_2_datarow_type': 'google_price',
            'datarow_2_tensor_type': 'goog_lin_reg_avg100day',
            'version': '1.0',
            'reset': False,
            'checkpoint': False,
            'limitLoad': False,
            'cleanData': True,
            'verbose': False,
            'rowLimit': 125,
            'validatePrct': 0.30,
            'trainPrct': 0.70,
            'randomSeed': False,
            'trainStepsMultiplier': 1,
            'learning_rate': 0.00000001,
            'log_reg_positive_result': 0.00,
            'lin_reg_positive_result': 0.50,
            'model_type': 'linear_regression',
            'loader': 'load_csv_data',
            'feature_type': 'goog_stock_sma100',
            'logPrint': 50,
            'eval_type': 'goog_stock_sma100',

            'files': {
                'file1': {'name': dataDir + "/spy.csv.xls", 'appendCols': [{'Symbol': 'spy', 'idx': '6'}]},
            }
        },

        "goog_log_reg":
        {
            'type': 'csv',
            'data_2_datarow_type': 'google_price2',
            'datarow_2_tensor_type': 'goog_log_reg',
            'version': '1.0',
            'reset': False,
            'checkpoint': False,
            'limitLoad': False,
            'cleanData': True,
            'verbose': False,
            'rowLimit': 125,
            'validatePrct': 0.50,
            'trainPrct': 0.70,
            'randomSeed': False,
            'trainStepsMultiplier': 1,
            'learning_rate': 0.0000000000000001,
            'log_reg_positive_result': 1.00,
            'lin_reg_positive_result': 0.00,
            'model_type': 'logistic_regression',
            'loader': 'load_csv_data',
            'feature_type': 'goog_log_reg',
            'logPrint': 50,
            'eval_type': '',

            'files': {
                'file1': {'name': dataDir + "/ivv.csv.xls", 'appendCols': [{'Symbol': 'ivv', 'idx': '6'}]},
            }
        },

        "weight_age_lin_reg":
        {
            'type': 'csv',
            'data_2_datarow_type': 'weight_age',
            'datarow_2_tensor_type': 'weight_age_lin_reg_blood_fat',
            'version': '1.0',
            'reset': False,
            'checkpoint': False,
            'limitLoad': False,
            'cleanData': True,
            'verbose': True,
            'rowLimit': 25,
            'validatePrct': 0.30,
            'trainPrct': 1.00,
            'randomSeed': False,
            'trainStepsMultiplier': 100,
            'learning_rate': 0.0001,
            'log_reg_positive_result': 0.00,
            'lin_reg_positive_result': 0.50,
            'model_type': 'linear_regression',
            'loader': 'load_csv_data',
            'feature_type': 'weight_age',
            'logPrint': 500,
            'eval_type': 'weight_age',

            'files': {
                'file1': {'name': dataDir + "/weight_age.csv.xls", 'appendCols': []}
            }
        },

        "blue_ice_log_reg":
        {
            # Apache Log File => alf
            'type': 'alf',
            'data_2_datarow_type': 'blue_ice',
            'datarow_2_tensor_type': 'blue_ice_log_reg_hack_attack',
            'version': '1.0',
            'reset': False,
            'checkpoint': True,
            'limitLoad': False,
            'cleanData': True,
            'verbose': False,
            'rowLimit': 25,
            'validatePrct': 0.20,
            'trainPrct': 0.80,
            'randomSeed': False,
            'trainStepsMultiplier': 20,
            'learning_rate': 0.000001,
            'log_reg_positive_result': 0.50,
            'lin_reg_positive_result': 0.50,
            'model_type': 'logistic_regression',
            'loader': 'load_apache_log_data',
            'feature_type': 'blue_ice_log_reg',
            'logPrint': 500,
            'checkpointSave': 500,
            'eval_type': 'blue_ice',

            'files': {
                'file1': {'name': dataDir + "/access_logs/access.log.1", 'appendCols': []},
                'file2': {'name': dataDir + "/access_logs/access.log.2", 'appendCols': []},
                'file3': {'name': dataDir + "/access_logs/access.log.3", 'appendCols': []},
                'file4': {'name': dataDir + "/access_logs/access.log.4", 'appendCols': []},
                'file5': {'name': dataDir + "/access_logs/access.log.5", 'appendCols': []},
                'file6': {'name': dataDir + "/access_logs/access.log.6", 'appendCols': []},
                'file7': {'name': dataDir + "/access_logs/access.log.7", 'appendCols': []},
                'file8': {'name': dataDir + "/access_logs/access.log.8", 'appendCols': []},
                'file9': {'name': dataDir + "/access_logs/access.log.9", 'appendCols': []},
                'file10': {'name': dataDir + "/access_logs/access.log.10", 'appendCols': []},
                'file11': {'name': dataDir + "/access_logs/access.log.11", 'appendCols': []},
                'file12': {'name': dataDir + "/access_logs/access.log.12", 'appendCols': []},
                'file13': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log", 'appendCols': []},
                'file14': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log.1", 'appendCols': []},
                'file15': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log.2", 'appendCols': []},
                'file16': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log.3", 'appendCols': []},
                'file17': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log.4", 'appendCols': []},
                'file18': {'name': dataDir + "/other_vhosts_access/other_vhosts_access.log.5", 'appendCols': []},
            },

            'extras': [
                '_websvc.php',
                '_websvcclient.php',
                '_scripts_utils_Log.js',
                '_scripts_utils_md5.js',
                '_scripts_utils_BufferedReader.js',
                '_scripts_webutils_MmgMd5.js',
                '_scripts_webutils_MsNewsSrchWebSvcClient.js',
                '_scripts_webutils_SaWebSvcClient.js',
                '_scripts_webutils_YhStockQuoteWebSvcClient.js',
                '_scripts_webutils_MsTxtSntWebSvcClient.js',
                '_scripts_stockanalytics_StockAnalytics.js',
                '_scripts_models_ModelsConfig.js',
                '_scripts_models_ModelArticlesCount.js',
                '_scripts_models_ModelArticlesGetAllActive.js',
                '_scripts_models_ModelDataSourcesCount.js',
                '_scripts_models_ModelStockQuotesCount.js',
                '_scripts_models_ModelDataSourcesUsageCount.js',
                '_scripts_models_ModelTargetSearchTextCount.js',
                '_scripts_models_ModelTargetSymbolsCount.js',
                '_scripts_models_ModelTextRefinementCount.js',
                '_scripts_models_ModelDataSourcesUsageSummary.js',
                '_scripts_models_ModelDataSourcesGetAllActive.js',
                '_scripts_models_ModelTargetSearchTextGetAllActive.js',
                '_scripts_models_ModelStockQuotesGetAllActive.js',
                '_scripts_models_ModelTargetSymbolsGetAllActive.js',
                '_scripts_models_ModelTextRefinementGetAllActive.js',
                '_scripts_views_ViewArticlesCount.js',
                '_scripts_views_ViewArticlesGetAllActive.js',
                '_scripts_views_ViewDataSourcesCount.js',
                '_scripts_views_ViewStockQuotesCount.js',
                '_scripts_views_ViewDataSourcesUsageCount.js',
                '_scripts_views_ViewTargetSearchTextCount.js',
                '_scripts_views_ViewTargetSymbolsCount.js',
                '_scripts_views_ViewTextRefinementCount.js',
                '_scripts_views_ViewDataSourcesUsageSummary.js',
                '_scripts_views_ViewDataSourcesGetAllActive.js',
                '_scripts_views_ViewStockQuotesGetAllActive.js',
                '_scripts_views_ViewTargetSearchTextGetAllActive.js',
                '_scripts_views_ViewTargetSymbolsGetAllActive.js',
                '_scripts_views_ViewTextRefinementGetAllActive.js',
                '_scripts_controllers_ControllerArticlesCount.js',
                '_scripts_controllers_ControllerDataSourcesCount.js',
                '_scripts_controllers_ControllerArticlesGetAllActive.js',
                '_scripts_controllers_ControllerStockQuotesCount.js',
                '_scripts_controllers_ControllerDataSourcesUsageCount.js',
                '_scripts_controllers_ControllerTargetSearchTextCount.js',
                '_scripts_controllers_ControllerTargetSymbolsCount.js',
                '_scripts_controllers_ControllerDataSourcesUsageSummary.js',
                '_scripts_controllers_ControllerTextRefinementCount.js',
                '_scripts_controllers_ControllerTargetSearchTextGetAllActive.js',
                '_scripts_controllers_ControllerDataSourcesGetAllActive.js',
                '_scripts_controllers_ControllerStockQuotesGetAllActive.js',
                '_scripts_controllers_ControllerTargetSymbolsGetAllActive.js',
                '_scripts_controllers_ControllerTextRefinementGetAllActive.js'
            ]
        }
    }

    run(exes["blue_ice_log_reg"]);

# edef

if __name__ == '__main__':
    main(sys.argv)
# eif
